%global pkg nvidia-open

%global kernel 6.12.0-101.el10
%global baserelease 1
%global upstream 570.169

%global debug_package %{nil}

# Do not set system compiler flags
%undefine _auto_set_build_flags

%global __spec_install_post \
  %{?__debug_package:%{__debug_install_post}} \
  %{__arch_install_post} \
  %{__os_install_post} \
  %{__mod_compress_install_post}

%global __mod_compress_install_post find %{buildroot}/lib/modules -type f -name \*.ko -exec xz  --compress --check=crc32 --lzma2=dict=1MiB \{\} \\;


Name:             kmod-%{pkg}-570
Epoch:            1
Version:          %(echo %{kernel} | sed 's/-/~/g; s/\.el.*$//g')
Release:          %{upstream}~%{baserelease}%{?dist}
Summary:          Kernel module (kmod) for NVIDIA Turing (GTX 16/RTX 20) and newer GPUs - PB R570

License:          GPL-2.0-only and MIT
URL:              https://github.com/NVIDIA/open-gpu-kernel-modules

Source0:          %{url}/archive/%{version}/open-gpu-kernel-modules-%{upstream}.tar.gz
Patch1:           0001-kernel-open-nvidia-drm-Enable-kernel-mode-setting-by.patch

ExclusiveArch:    x86_64 aarch64

BuildRequires:    elfutils-libelf-devel
BuildRequires:    gcc
BuildRequires:    kernel-rpm-macros
BuildRequires:    kmod
BuildRequires:    make
BuildRequires:    redhat-rpm-config
BuildRequires:    xz

BuildRequires:    kernel-abi-stablelists = %{kernel}
BuildRequires:    kernel-devel-uname-r = %{kernel}.%{_arch}

Requires:         kernel-uname-r >= %{kernel}.%{_arch}

Provides:         installonlypkg(kernel-module)
Provides:         kernel-modules >= %{kernel}.%{_arch}

Requires(posttrans): %{_sbindir}/depmod
Requires(postun):    %{_sbindir}/depmod

Requires(posttrans): %{_sbindir}/weak-modules
Requires(postun):    %{_sbindir}/weak-modules

Requires(posttrans): %{_bindir}/sort
Requires(postun):    %{_bindir}/sort


%if %{epoch}
Obsoletes:        %{name} < %{epoch}:
%endif
Obsoletes:        %{name} = %{?epoch:%{epoch}:}%{version}


BuildRequires:    gcc-c++
BuildRequires:    git-core

Conflicts:        kmod-nvidia

Provides:         kmod-%{pkg}
Conflicts:        kmod-%{pkg}


%description
This packages provides the open NVIDIA Turing (GTX 16/RTX 20) and newer models
of GPU hardware kernel module (%{pkg}) Production Branch R570.


%prep
%autosetup -n open-gpu-kernel-modules-%{upstream} -S git_am


%build
export SYSSRC=/usr/src/kernels/%{kernel}.%{_arch}
export NV_EXCLUDE_KERNEL_MODULES=nvidia-peermem
%{__make} %{?_smp_mflags} M=$PWD modules


%install
%{__install} -D -t %{buildroot}/lib/modules/%{kernel}.%{_arch}/extra/%{pkg} kernel-open/nvidia-drm.ko kernel-open/nvidia-modeset.ko kernel-open/nvidia-uvm.ko kernel-open/nvidia.ko

# Make .ko objects temporarily executable for automatic stripping
find %{buildroot}/lib/modules -type f -name \*.ko -exec chmod u+x \{\} \+


%clean
%{__rm} -rf %{buildroot}


%triggerin -- %{name}
mkdir -p %{_localstatedir}/lib/rpm-state/sig-kmods
printf '%s\n' "/lib/modules/%{kernel}.%{_arch}/extra/%{pkg}/nvidia-drm.ko.xz" >> %{_localstatedir}/lib/rpm-state/sig-kmods/weak-modules
printf '%s\n' "/lib/modules/%{kernel}.%{_arch}/extra/%{pkg}/nvidia-modeset.ko.xz" >> %{_localstatedir}/lib/rpm-state/sig-kmods/weak-modules
printf '%s\n' "/lib/modules/%{kernel}.%{_arch}/extra/%{pkg}/nvidia-uvm.ko.xz" >> %{_localstatedir}/lib/rpm-state/sig-kmods/weak-modules
printf '%s\n' "/lib/modules/%{kernel}.%{_arch}/extra/%{pkg}/nvidia.ko.xz" >> %{_localstatedir}/lib/rpm-state/sig-kmods/weak-modules


%triggerun -- %{name}
mkdir -p %{_localstatedir}/lib/rpm-state/sig-kmods
printf '%s\n' "/lib/modules/%{kernel}.%{_arch}/extra/%{pkg}/nvidia-drm.ko.xz" >> %{_localstatedir}/lib/rpm-state/sig-kmods/weak-modules
printf '%s\n' "/lib/modules/%{kernel}.%{_arch}/extra/%{pkg}/nvidia-modeset.ko.xz" >> %{_localstatedir}/lib/rpm-state/sig-kmods/weak-modules
printf '%s\n' "/lib/modules/%{kernel}.%{_arch}/extra/%{pkg}/nvidia-uvm.ko.xz" >> %{_localstatedir}/lib/rpm-state/sig-kmods/weak-modules
printf '%s\n' "/lib/modules/%{kernel}.%{_arch}/extra/%{pkg}/nvidia.ko.xz" >> %{_localstatedir}/lib/rpm-state/sig-kmods/weak-modules


%preun
mkdir -p %{_localstatedir}/lib/rpm-state/sig-kmods
rpm -ql kmod-%{pkg}-%{?epoch:%{epoch}:}%{version}-%{release}.%{_arch} | grep '/lib/modules/%{kernel}.%{_arch}/.*\.ko.xz$' >> %{_localstatedir}/lib/rpm-state/sig-kmods/weak-modules-remove


%postun
if [ -f %{_localstatedir}/lib/rpm-state/sig-kmods/weak-modules ]
then
    while read -r MODULE
    do
        if [ -f "$MODULE" ]
        then
            printf '%s\n' "$MODULE" >> %{_localstatedir}/lib/rpm-state/sig-kmods/weak-modules-add
        fi
    done < %{_localstatedir}/lib/rpm-state/sig-kmods/weak-modules
    rm -f %{_localstatedir}/lib/rpm-state/sig-kmods/weak-modules
fi
if [ -f %{_localstatedir}/lib/rpm-state/sig-kmods/weak-modules-remove ]
then
    modules=( $(cat %{_localstatedir}/lib/rpm-state/sig-kmods/weak-modules-remove | sort -u -V -t '/' -k 5 -k 4,4) )
    rm -f %{_localstatedir}/lib/rpm-state/sig-kmods/weak-modules-remove
    if [ -f %{_localstatedir}/lib/rpm-state/sig-kmods/weak-modules-add ]
    then
        printf '%s\n' "${modules[@]}" | %{_sbindir}/weak-modules --remove-modules --no-initramfs
    else
        printf '%s\n' "${modules[@]}" | %{_sbindir}/weak-modules --remove-modules
    fi
fi
if [ -f %{_localstatedir}/lib/rpm-state/sig-kmods/weak-modules-add ]
then
    modules=( $(cat %{_localstatedir}/lib/rpm-state/sig-kmods/weak-modules-add | sort -u -V -t '/' -k 5 -k 4,4) )
    rm -f %{_localstatedir}/lib/rpm-state/sig-kmods/weak-modules-add
    printf '%s\n' "${modules[@]}" | %{_sbindir}/weak-modules --add-modules
fi
if [ -d %{_localstatedir}/lib/rpm-state/sig-kmods ]
then
    rmdir --ignore-fail-on-non-empty %{_localstatedir}/lib/rpm-state/sig-kmods
fi


%posttrans
if [ -f %{_localstatedir}/lib/rpm-state/sig-kmods/weak-modules ]
then
    while read -r MODULE
    do
        if [ -f "$MODULE" ]
        then
            printf '%s\n' "$MODULE" >> %{_localstatedir}/lib/rpm-state/sig-kmods/weak-modules-add
        fi
    done < %{_localstatedir}/lib/rpm-state/sig-kmods/weak-modules
    rm -f %{_localstatedir}/lib/rpm-state/sig-kmods/weak-modules
fi
if [ -f %{_localstatedir}/lib/rpm-state/sig-kmods/weak-modules-add ]
then
    modules=( $(cat %{_localstatedir}/lib/rpm-state/sig-kmods/weak-modules-add | sort -u -V -t '/' -k 5 -k 4,4) )
    rm -f %{_localstatedir}/lib/rpm-state/sig-kmods/weak-modules-add
    printf '%s\n' "${modules[@]}" | %{_sbindir}/weak-modules --add-modules
fi
if [ -d %{_localstatedir}/lib/rpm-state/sig-kmods ]
then
    rmdir --ignore-fail-on-non-empty %{_localstatedir}/lib/rpm-state/sig-kmods
fi


%files
%defattr(644,root,root,755)
/lib/modules/%{kernel}.%{_arch}
%license COPYING
%doc README.md


%changelog
* Thu Jul 17 2025 Kmods SIG <sig-kmods@centosproject.org> - 1:6.12.0~101-570.169~1
- kABI tracking kmod package (kernel >= 6.12.0-101.el10)
